@@ -1,5 +1,5 @@
 /* gifdiff.c - Gifdiff compares GIF images for identical appearance.
-   Copyright (C) 1998-2014 Eddie Kohler, ekohler@gmail.com
+   Copyright (C) 1998-2018 Eddie Kohler, ekohler@gmail.com
    This file is part of gifdiff, in the gifsicle package.
 
    Gifdiff is free software. It is distributed under the GNU Public License,
@@ -118,11 +118,10 @@ apply_image(int is_second, Gif_Stream *gfs, int imageno, uint16_t background)
   Gif_Colormap *gfcm = gfi->local ? gfi->local : gfs->global;
 
   /* set up colormap */
-  for (i = 0; i < 256; i++)
+  for (i = 0; i < gfcm->ncol; ++i)
+    map[i] = gfcm->col[i].pixel;
+  for (i = gfcm->ncol; i < 256; ++i)
     map[i] = 1;
-  if (gfs)
-    for (i = 0; i < gfcm->ncol; i++)
-      map[i] = gfcm->col[i].pixel;
   if (gfi->transparent >= 0 && gfi->transparent < 256)
     map[gfi->transparent] = TRANSP;
 
@@ -249,12 +248,6 @@ compare(Gif_Stream *s1, Gif_Stream *s2)
   Gif_CalculateScreenSize(s1, 0);
   Gif_CalculateScreenSize(s2, 0);
 
-  if (s1->nimages != s2->nimages
-      && (s1->nimages == 0 || s2->nimages == 0)) {
-    different("frame counts differ: <#%d >#%d", s1->nimages, s2->nimages);
-    return DIFFERENT;
-  }
-
   if (s1->screen_width != s2->screen_width
       || s1->screen_height != s2->screen_height) {
     different("screen sizes differ: <%dx%d >%dx%d", s1->screen_width,
@@ -269,6 +262,14 @@ compare(Gif_Stream *s1, Gif_Stream *s2)
     return DIFFERENT;
   }
 
+  if (s1->nimages == 0 || s2->nimages == 0) {
+    if (s1->nimages != s2->nimages) {
+      different("frame counts differ: <#%d >#%d", s1->nimages, s2->nimages);
+      return DIFFERENT;
+    } else
+      return SAME;
+  }
+
   /* Create arrays for the image data */
   screen_width = s1->screen_width;
   screen_height = s1->screen_height;
@@ -458,7 +459,7 @@ gifread_error(Gif_Stream* gfs, Gif_Image* gfi,
   static int same_error_count = 0;
   int which_image = Gif_ImageNumber(gfs, gfi);
   const char *filename = gfs->landmark;
-  if (gfs && which_image < 0)
+  if (which_image < 0)
     which_image = gfs->nimages;
 
   if (gifread_error_count == 0) {
@@ -559,7 +560,7 @@ main(int argc, char *argv[])
 
      case VERSION_OPT:
       printf("gifdiff (LCDF Gifsicle) %s\n", VERSION);
-      printf("Copyright (C) 1998-2014 Eddie Kohler\n\
+      printf("Copyright (C) 1998-2018 Eddie Kohler\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
