@@ -1,6 +1,6 @@
 /* -*- mode: c; c-basic-offset: 2 -*- */
 /* gifwrite.c - Functions to write GIFs.
-   Copyright (C) 1997-2014 Eddie Kohler, ekohler@gmail.com
+   Copyright (C) 1997-2018 Eddie Kohler, ekohler@gmail.com
    This file is part of the LCDF GIF library.
 
    The LCDF GIF library is free software. It is distributed under the GNU
@@ -70,7 +70,7 @@ typedef struct Gif_CodeTable {
 } Gif_CodeTable;
 
 
-typedef struct Gif_Writer {
+struct Gif_Writer {
   FILE *f;
   uint8_t *v;
   uint32_t pos;
@@ -80,9 +80,10 @@ typedef struct Gif_Writer {
   int local_size;
   int errors;
   int cleared;
+  Gif_CodeTable code_table;
   void (*byte_putter)(uint8_t, struct Gif_Writer *);
-  void (*block_putter)(const uint8_t *, uint16_t, struct Gif_Writer *);
-} Gif_Writer;
+  void (*block_putter)(const uint8_t *, size_t, struct Gif_Writer *);
+};
 
 
 #define gifputbyte(b, grr)	((*grr->byte_putter)(b, grr))
@@ -103,9 +104,9 @@ file_byte_putter(uint8_t b, Gif_Writer *grr)
 }
 
 static void
-file_block_putter(const uint8_t *block, uint16_t size, Gif_Writer *grr)
+file_block_putter(const uint8_t *block, size_t size, Gif_Writer *grr)
 {
-  if (fwrite(block, 1, size, grr->f) != (size_t) size)
+  if (fwrite(block, 1, size, grr->f) != size)
     grr->errors = 1;
 }
 
@@ -124,7 +125,7 @@ memory_byte_putter(uint8_t b, Gif_Writer *grr)
 }
 
 static void
-memory_block_putter(const uint8_t *data, uint16_t len, Gif_Writer *grr)
+memory_block_putter(const uint8_t *data, size_t len, Gif_Writer *grr)
 {
   while (grr->pos + len >= grr->cap) {
     grr->cap = (grr->cap ? grr->cap * 2 : 1024);
@@ -138,13 +139,38 @@ memory_block_putter(const uint8_t *data, uint16_t len, Gif_Writer *grr)
 
 
 static int
-gfc_init(Gif_CodeTable *gfc)
+gif_writer_init(Gif_Writer* grr, FILE* f, const Gif_CompressInfo* gcinfo)
 {
-  gfc->nodes = Gif_NewArray(Gif_Node, NODES_SIZE);
-  gfc->links = Gif_NewArray(Gif_Node *, LINKS_SIZE);
-  return gfc->nodes && gfc->links;
+  grr->f = f;
+  grr->v = NULL;
+  grr->pos = grr->cap = 0;
+  if (gcinfo)
+    grr->gcinfo = *gcinfo;
+  else
+    Gif_InitCompressInfo(&grr->gcinfo);
+  grr->errors = 0;
+  grr->cleared = 0;
+  grr->code_table.nodes = Gif_NewArray(Gif_Node, NODES_SIZE);
+  grr->code_table.links = Gif_NewArray(Gif_Node*, LINKS_SIZE);
+  if (f) {
+    grr->byte_putter = file_byte_putter;
+    grr->block_putter = file_block_putter;
+  } else {
+    grr->byte_putter = memory_byte_putter;
+    grr->block_putter = memory_block_putter;
+  }
+  return grr->code_table.nodes && grr->code_table.links;
 }
 
+static void
+gif_writer_cleanup(Gif_Writer* grr)
+{
+  Gif_DeleteArray(grr->v);
+  Gif_DeleteArray(grr->code_table.nodes);
+  Gif_DeleteArray(grr->code_table.links);
+}
+
+
 static inline void
 gfc_clear(Gif_CodeTable *gfc, Gif_Code clear_code)
 {
@@ -249,9 +275,10 @@ gif_line_endpos(Gif_Image *gfi, unsigned pos)
 
 static int
 write_compressed_data(Gif_Image *gfi,
-		      int min_code_bits, Gif_CodeTable *gfc, Gif_Writer *grr)
+		      int min_code_bits, Gif_Writer *grr)
 {
-  uint8_t stack_buffer[232];
+  Gif_CodeTable* gfc = &grr->code_table;
+  uint8_t stack_buffer[512 - 24];
   uint8_t *buf = stack_buffer;
   unsigned bufpos = 0;
   unsigned bufcap = sizeof(stack_buffer) * 8;
@@ -261,33 +288,31 @@ write_compressed_data(Gif_Image *gfi,
   unsigned line_endpos;
   const uint8_t *imageline;
 
-  Gif_Node *work_node;
-  unsigned run;
+  unsigned run = 0;
 #define RUN_EWMA_SHIFT 4
 #define RUN_EWMA_SCALE 19
 #define RUN_INV_THRESH ((unsigned) (1 << RUN_EWMA_SCALE) / 3000)
-  unsigned run_ewma;
+  unsigned run_ewma = 0;
+  Gif_Node *work_node;
   Gif_Node *next_node;
   Gif_Code next_code = 0;
   Gif_Code output_code;
-#define CUR_BUMP_CODE (1 << cur_code_bits)
   uint8_t suffix;
 
   int cur_code_bits;
 
   /* Here we go! */
   gifputbyte(min_code_bits, grr);
-#define CLEAR_CODE	((Gif_Code) (1 << min_code_bits))
-#define EOI_CODE	((Gif_Code) (CLEAR_CODE + 1))
+#define CLEAR_CODE      ((Gif_Code) (1 << min_code_bits))
+#define EOI_CODE        ((Gif_Code) (CLEAR_CODE + 1))
+#define CUR_BUMP_CODE   (1 << cur_code_bits)
   grr->cleared = 0;
 
   cur_code_bits = min_code_bits + 1;
   /* next_code set by first runthrough of output clear_code */
-  GIF_DEBUG(("clear(%d) eoi(%d) bits(%d)", CLEAR_CODE, EOI_CODE, cur_code_bits));
+  GIF_DEBUG(("clear(%d) eoi(%d) bits(%d) ", CLEAR_CODE, EOI_CODE, cur_code_bits));
 
   work_node = 0;
-  run = 0;
-  run_ewma = 1 << RUN_EWMA_SCALE;
   output_code = CLEAR_CODE;
   /* Because output_code is clear_code, we'll initialize next_code, et al.
      below. */
@@ -300,11 +325,11 @@ write_compressed_data(Gif_Image *gfi,
 
     /*****
      * Output 'output_code' to the memory buffer. */
-    if (bufpos + cur_code_bits >= bufcap) {
+    if (bufpos + 32 >= bufcap) {
       unsigned ncap = bufcap * 2 + (24 << 3);
       uint8_t *nbuf = Gif_NewArray(uint8_t, ncap >> 3);
       if (!nbuf)
-        return 0;
+        goto error;
       memcpy(nbuf, buf, bufcap >> 3);
       if (buf != stack_buffer)
         Gif_DeleteArray(buf);
@@ -313,16 +338,20 @@ write_compressed_data(Gif_Image *gfi,
     }
 
     {
-      unsigned startpos = bufpos;
+      unsigned endpos = bufpos + cur_code_bits;
       do {
         if (bufpos & 7)
           buf[bufpos >> 3] |= output_code << (bufpos & 7);
-        else
-          buf[bufpos >> 3] = output_code >> (bufpos - startpos);
+        else if (bufpos & 0x7FF)
+          buf[bufpos >> 3] = output_code >> (bufpos - endpos + cur_code_bits);
+        else {
+          buf[bufpos >> 3] = 255;
+          endpos += 8;
+        }
 
         bufpos += 8 - (bufpos & 7);
-      } while (bufpos < startpos + cur_code_bits);
-      bufpos = startpos + cur_code_bits;
+      } while (bufpos < endpos);
+      bufpos = endpos;
     }
 
 
@@ -334,17 +363,30 @@ write_compressed_data(Gif_Image *gfi,
       cur_code_bits = min_code_bits + 1;
       next_code = EOI_CODE + 1;
       run_ewma = 1 << RUN_EWMA_SCALE;
+      run = 0;
       gfc_clear(gfc, CLEAR_CODE);
       clear_pos = clear_bufpos = 0;
 
-      GIF_DEBUG(("clear"));
+      GIF_DEBUG(("clear "));
 
     } else if (output_code == EOI_CODE)
       break;
 
-    else if (next_code > CUR_BUMP_CODE && cur_code_bits < GIF_MAX_CODE_BITS)
-      /* bump up compression size */
-      ++cur_code_bits;
+    else {
+      if (next_code > CUR_BUMP_CODE && cur_code_bits < GIF_MAX_CODE_BITS)
+        /* bump up compression size */
+        ++cur_code_bits;
+
+      /* Adjust current run length average. */
+      run = (run << RUN_EWMA_SCALE) + (1 << (RUN_EWMA_SHIFT - 1));
+      if (run < run_ewma)
+        run_ewma -= (run_ewma - run) >> RUN_EWMA_SHIFT;
+      else
+        run_ewma += (run - run_ewma) >> RUN_EWMA_SHIFT;
+
+      /* Reset run length. */
+      run = !!work_node;
+    }
 
 
     /*****
@@ -363,96 +405,83 @@ write_compressed_data(Gif_Image *gfi,
         line_endpos += gfi->width;
       }
 
-      if (!next_node) {
-	/* Output the current code. */
-	if (next_code < GIF_MAX_CODE) {
-          gfc_define(gfc, work_node, suffix, next_code);
-          next_code++;
-	} else
-	  next_code = GIF_MAX_CODE + 1; /* to match "> CUR_BUMP_CODE" above */
+      if (next_node) {
+        work_node = next_node;
+        ++run;
+        continue;
+      }
 
-        /* Check whether to clear table. */
-        if (next_code > 4094) {
-          int do_clear = grr->gcinfo.flags & GIF_WRITE_EAGER_CLEAR;
+      /* Output the current code. */
+      if (next_code < GIF_MAX_CODE) {
+        gfc_define(gfc, work_node, suffix, next_code);
+        next_code++;
+      } else
+        next_code = GIF_MAX_CODE + 1; /* to match "> CUR_BUMP_CODE" above */
 
-          if (!do_clear) {
-            unsigned pixels_left = gfi->width * gfi->height - pos;
-            if (pixels_left) {
-              /* Always clear if run_ewma gets small relative to
-                 min_code_bits. Otherwise, clear if #images/run is smaller
-                 than an empirical threshold, meaning it will take more than
-                 3000 or so average runs to complete the image. */
-              if (run_ewma < ((36U << RUN_EWMA_SCALE) / min_code_bits)
-                  || pixels_left > UINT_MAX / RUN_INV_THRESH
-                  || run_ewma < pixels_left * RUN_INV_THRESH)
-                do_clear = 1;
-            }
-          }
+      /* Check whether to clear table. */
+      if (next_code > 4094) {
+        int do_clear = grr->gcinfo.flags & GIF_WRITE_EAGER_CLEAR;
 
-          if ((do_clear || run < 7) && !clear_pos) {
-            clear_pos = pos - (run + 1);
-            clear_bufpos = bufpos;
-          } else if (!do_clear && run > 50)
-            clear_pos = clear_bufpos = 0;
-
-          if (do_clear) {
-            GIF_DEBUG(("rewind %u pixels/%d bits", pos - clear_pos, bufpos + cur_code_bits - clear_bufpos));
-            output_code = CLEAR_CODE;
-            pos = clear_pos;
-            imageline = gif_imageline(gfi, pos);
-            line_endpos = gif_line_endpos(gfi, pos);
-            bufpos = clear_bufpos;
-            buf[bufpos >> 3] &= (1 << (bufpos & 7)) - 1;
-            work_node = 0;
-            run = 0;
-            grr->cleared = 1;
-            goto found_output_code;
+        if (!do_clear) {
+          unsigned pixels_left = gfi->width * gfi->height - pos;
+          if (pixels_left) {
+            /* Always clear if run_ewma gets small relative to
+               min_code_bits. Otherwise, clear if #images/run is smaller
+               than an empirical threshold, meaning it will take more than
+               3000 or so average runs to complete the image. */
+            if (run_ewma < ((36U << RUN_EWMA_SCALE) / min_code_bits)
+                || pixels_left > UINT_MAX / RUN_INV_THRESH
+                || run_ewma < pixels_left * RUN_INV_THRESH)
+              do_clear = 1;
           }
         }
 
-        /* Adjust current run length average. */
-        run = (run << RUN_EWMA_SCALE) + (1 << (RUN_EWMA_SHIFT - 1));
-        if (run < run_ewma)
-          run_ewma -= (run_ewma - run) >> RUN_EWMA_SHIFT;
-        else
-          run_ewma += (run - run_ewma) >> RUN_EWMA_SHIFT;
+        if ((do_clear || run < 7) && !clear_pos) {
+          clear_pos = pos - (run + 1);
+          clear_bufpos = bufpos;
+        } else if (!do_clear && run > 50)
+          clear_pos = clear_bufpos = 0;
 
-	/* Output the current code. */
-	output_code = work_node->code;
-	work_node = &gfc->nodes[suffix];
-	run = 1;
-	goto found_output_code;
+        if (do_clear) {
+          GIF_DEBUG(("rewind %u pixels/%d bits ", pos - clear_pos, bufpos + cur_code_bits - clear_bufpos));
+          output_code = CLEAR_CODE;
+          pos = clear_pos;
+          imageline = gif_imageline(gfi, pos);
+          line_endpos = gif_line_endpos(gfi, pos);
+          bufpos = clear_bufpos;
+          buf[bufpos >> 3] &= (1 << (bufpos & 7)) - 1;
+          work_node = 0;
+          grr->cleared = 1;
+          goto found_output_code;
+        }
       }
 
-      work_node = next_node;
-      ++run;
+      output_code = work_node->code;
+      work_node = &gfc->nodes[suffix];
+      goto found_output_code;
     }
 
     /* Ran out of data if we get here. */
     output_code = (work_node ? work_node->code : EOI_CODE);
     work_node = 0;
-    run = 0;
 
    found_output_code: ;
   }
 
   /* Output memory buffer to stream. */
-  {
-    unsigned outpos = 0;
-    bufpos = (bufpos + 7) >> 3;
-    while (outpos < bufpos) {
-      unsigned w = (bufpos - outpos > 255 ? 255 : bufpos - outpos);
-      gifputbyte(w, grr);
-      gifputblock(buf + outpos, w, grr);
-      outpos += w;
-    }
-    gifputbyte(0, grr);
-  }
+  bufpos = (bufpos + 7) >> 3;
+  buf[(bufpos - 1) & 0xFFFFFF00] = (bufpos - 1) & 0xFF;
+  buf[bufpos] = 0;
+  gifputblock(buf, bufpos + 1, grr);
 
   if (buf != stack_buffer)
     Gif_DeleteArray(buf);
-
   return 1;
+
+ error:
+  if (buf != stack_buffer)
+    Gif_DeleteArray(buf);
+  return 0;
 }
 
 
@@ -511,8 +540,9 @@ save_compression_result(Gif_Image *gfi, Gif_Writer *grr, int ok)
     if (gfi->compressed)
       (*gfi->free_compressed)((void *) gfi->compressed);
     if (ok) {
-      gfi->compressed = grr->v;
       gfi->compressed_len = grr->pos;
+      gfi->compressed_errors = 0;
+      gfi->compressed = grr->v;
       gfi->free_compressed = Gif_Free;
       grr->v = 0;
       grr->cap = 0;
@@ -529,45 +559,31 @@ Gif_FullCompressImage(Gif_Stream *gfs, Gif_Image *gfi,
   int ok = 0;
   uint8_t min_code_bits;
   Gif_Writer grr;
-  Gif_CodeTable gfc;
 
-  gfc_init(&gfc);
-
-  grr.v = NULL;
-  grr.pos = grr.cap = 0;
-  grr.byte_putter = memory_byte_putter;
-  grr.block_putter = memory_block_putter;
-  if (gcinfo)
-    grr.gcinfo = *gcinfo;
-  else
-    Gif_InitCompressInfo(&grr.gcinfo);
-  grr.global_size = get_color_table_size(gfs, 0, &grr);
-  grr.local_size = get_color_table_size(gfs, gfi, &grr);
-  grr.errors = 0;
-
-  if (!gfc.nodes || !gfc.links) {
+  if (!gif_writer_init(&grr, NULL, gcinfo)) {
     if (!(grr.gcinfo.flags & GIF_WRITE_SHRINK))
       Gif_ReleaseCompressedImage(gfi);
     goto done;
   }
 
+  grr.global_size = get_color_table_size(gfs, 0, &grr);
+  grr.local_size = get_color_table_size(gfs, gfi, &grr);
+
   min_code_bits = calculate_min_code_bits(gfi, &grr);
-  ok = write_compressed_data(gfi, min_code_bits, &gfc, &grr);
+  ok = write_compressed_data(gfi, min_code_bits, &grr);
   save_compression_result(gfi, &grr, ok);
 
   if ((grr.gcinfo.flags & (GIF_WRITE_OPTIMIZE | GIF_WRITE_EAGER_CLEAR))
       == GIF_WRITE_OPTIMIZE
       && grr.cleared && ok) {
     grr.gcinfo.flags |= GIF_WRITE_EAGER_CLEAR | GIF_WRITE_SHRINK;
-    if (write_compressed_data(gfi, min_code_bits, &gfc, &grr))
+    if (write_compressed_data(gfi, min_code_bits, &grr))
       save_compression_result(gfi, &grr, 1);
   }
 
  done:
-  Gif_DeleteArray(grr.v);
-  Gif_DeleteArray(gfc.nodes);
-  Gif_DeleteArray(gfc.links);
-  return grr.v != 0;
+  gif_writer_cleanup(&grr);
+  return ok;
 }
 
 
@@ -626,8 +642,7 @@ write_color_table(Gif_Colormap *gfcm, int totalcol, Gif_Writer *grr)
 
 
 static int
-write_image(Gif_Stream *gfs, Gif_Image *gfi, Gif_CodeTable *gfc,
-            Gif_Writer *grr)
+write_image(Gif_Stream *gfs, Gif_Image *gfi, Gif_Writer *grr)
 {
   uint8_t min_code_bits, packed = 0;
   grr->local_size = get_color_table_size(gfs, gfi, grr);
@@ -672,11 +687,11 @@ write_image(Gif_Stream *gfs, Gif_Image *gfi, Gif_CodeTable *gfc,
 
   } else if (!gfi->img) {
     Gif_UncompressImage(gfs, gfi);
-    write_compressed_data(gfi, min_code_bits, gfc, grr);
+    write_compressed_data(gfi, min_code_bits, grr);
     Gif_ReleaseUncompressedImage(gfi);
 
   } else
-    write_compressed_data(gfi, min_code_bits, gfc, grr);
+    write_compressed_data(gfi, min_code_bits, grr);
 
   return 1;
 }
@@ -810,29 +825,22 @@ write_generic_extension(Gif_Extension *gfex, Gif_Writer *grr)
   gifputbyte(0, grr);
 }
 
-
 static int
 write_gif(Gif_Stream *gfs, Gif_Writer *grr)
 {
+  Gif_Extension* gfex;
   int ok = 0;
   int i;
-  Gif_Image *gfi;
-  Gif_Extension *gfex = gfs->extensions;
-  Gif_CodeTable gfc;
-
-  gfc_init(&gfc);
-  if (!gfc.nodes || !gfc.links)
-    goto done;
 
   {
     uint8_t isgif89a = 0;
-    if (gfs->comment || gfs->loopcount > -1)
+    if (gfs->end_comment || gfs->end_extension_list || gfs->loopcount > -1)
       isgif89a = 1;
     for (i = 0; i < gfs->nimages && !isgif89a; i++) {
-      gfi = gfs->images[i];
-      if (gfi->identifier || gfi->transparent != -1 || gfi->disposal ||
-	  gfi->delay || gfi->comment)
-	isgif89a = 1;
+      Gif_Image* gfi = gfs->images[i];
+      if (gfi->identifier || gfi->transparent != -1 || gfi->disposal
+          || gfi->delay || gfi->comment || gfi->extension_list)
+        isgif89a = 1;
     }
     if (isgif89a)
       gifputblock((const uint8_t *)"GIF89a", 6, grr);
@@ -845,35 +853,19 @@ write_gif(Gif_Stream *gfs, Gif_Writer *grr)
   if (gfs->loopcount > -1)
     write_netscape_loop_extension(gfs->loopcount, grr);
 
-  for (i = 0; i < gfs->nimages; i++) {
-    Gif_Image *gfi = gfs->images[i];
-    while (gfex && gfex->position == i) {
-      write_generic_extension(gfex, grr);
-      gfex = gfex->next;
-    }
-    if (gfi->comment)
-      write_comment_extensions(gfi->comment, grr);
-    if (gfi->identifier)
-      write_name_extension(gfi->identifier, grr);
-    if (gfi->transparent != -1 || gfi->disposal || gfi->delay)
-      write_graphic_control_extension(gfi, grr);
-    if (!write_image(gfs, gfi, &gfc, grr))
+  for (i = 0; i < gfs->nimages; i++)
+    if (!Gif_IncrementalWriteImage(grr, gfs, gfs->images[i]))
       goto done;
-  }
 
-  while (gfex) {
+  for (gfex = gfs->end_extension_list; gfex; gfex = gfex->next)
     write_generic_extension(gfex, grr);
-    gfex = gfex->next;
-  }
-  if (gfs->comment)
-    write_comment_extensions(gfs->comment, grr);
+  if (gfs->end_comment)
+    write_comment_extensions(gfs->end_comment, grr);
 
   gifputbyte(';', grr);
   ok = 1;
 
  done:
-  Gif_DeleteArray(gfc.nodes);
-  Gif_DeleteArray(gfc.links);
   return ok;
 }
 
@@ -883,15 +875,56 @@ Gif_FullWriteFile(Gif_Stream *gfs, const Gif_CompressInfo *gcinfo,
 		  FILE *f)
 {
   Gif_Writer grr;
-  grr.f = f;
-  grr.byte_putter = file_byte_putter;
-  grr.block_putter = file_block_putter;
-  if (gcinfo)
-    grr.gcinfo = *gcinfo;
-  else
-    Gif_InitCompressInfo(&grr.gcinfo);
-  grr.errors = 0;
-  return write_gif(gfs, &grr);
+  int ok = gif_writer_init(&grr, f, gcinfo)
+           && write_gif(gfs, &grr);
+  gif_writer_cleanup(&grr);
+  return ok;
+}
+
+
+Gif_Writer*
+Gif_IncrementalWriteFileInit(Gif_Stream* gfs, const Gif_CompressInfo* gcinfo,
+                             FILE *f)
+{
+    Gif_Writer* grr = Gif_New(Gif_Writer);
+    if (!grr || !gif_writer_init(grr, f, gcinfo)) {
+        Gif_Delete(grr);
+        return NULL;
+    }
+    gifputblock((const uint8_t *)"GIF89a", 6, grr);
+    write_logical_screen_descriptor(gfs, grr);
+    if (gfs->loopcount > -1)
+        write_netscape_loop_extension(gfs->loopcount, grr);
+    return grr;
+}
+
+int
+Gif_IncrementalWriteImage(Gif_Writer* grr, Gif_Stream* gfs, Gif_Image* gfi)
+{
+    Gif_Extension *gfex;
+    for (gfex = gfi->extension_list; gfex; gfex = gfex->next)
+        write_generic_extension(gfex, grr);
+    if (gfi->comment)
+        write_comment_extensions(gfi->comment, grr);
+    if (gfi->identifier)
+        write_name_extension(gfi->identifier, grr);
+    if (gfi->transparent != -1 || gfi->disposal || gfi->delay)
+        write_graphic_control_extension(gfi, grr);
+    return write_image(gfs, gfi, grr);
+}
+
+int
+Gif_IncrementalWriteComplete(Gif_Writer* grr, Gif_Stream* gfs)
+{
+    Gif_Extension* gfex;
+    for (gfex = gfs->end_extension_list; gfex; gfex = gfex->next)
+        write_generic_extension(gfex, grr);
+    if (gfs->end_comment)
+        write_comment_extensions(gfs->end_comment, grr);
+    gifputbyte(';', grr);
+    gif_writer_cleanup(grr);
+    Gif_Delete(grr);
+    return 1;
 }
 
 
